@@ -17,6 +17,7 @@
 import doctest
 
 import testtools
+from testtools.compat import _b
 from testtools.matchers import (
     DocTestMatches,
     Equals,
@@ -46,19 +47,14 @@ class TestCommand(ResourcedTestCase):
         repo = cmd.repository_factory.initialise(ui.here)
         inserter = repo.get_inserter()
         inserter.startTestRun()
-        class Cases(ResourcedTestCase):
-            def failing(self):
-                self.fail('foo')
-            def ok(self):
-                pass
-        Cases('failing').run(inserter)
-        Cases('ok').run(inserter)
+        inserter.status(test_id='failing', test_status='fail')
+        inserter.status(test_id='ok', test_status='success')
         inserter.stopTestRun()
         self.assertEqual(1, cmd.execute())
         # We should have seen test outputs (of the failure) and summary data.
         self.assertEqual([
             ('results', Wildcard),
-            ('summary', False, 1, None, None, None, [('id', 0, None), ('failures', 1, None)])],
+            ('summary', False, 1, None, Wildcard, None, [('id', 0, None), ('failures', 1, None)])],
             ui.outputs)
         suite = ui.outputs[0][1]
         result = testtools.TestResult()
@@ -76,18 +72,14 @@ class TestCommand(ResourcedTestCase):
         repo = cmd.repository_factory.initialise(ui.here)
         inserter = repo.get_inserter()
         inserter.startTestRun()
-        class Cases(ResourcedTestCase):
-            def failing(self):
-                self.fail('foo')
-            def ok(self):
-                pass
-        Cases('failing').run(inserter)
-        Cases('ok').run(inserter)
+        inserter.status(test_id='failing', test_status='fail')
+        inserter.status(test_id='ok', test_status='success')
         inserter.stopTestRun()
         self.assertEqual(0, cmd.execute())
         self.assertEqual(1, len(ui.outputs))
         self.assertEqual('stream', ui.outputs[0][0])
-        self.assertThat(ui.outputs[0][1], DocTestMatches("""...test: ...failing
+        self.assertThat(ui.outputs[0][1].decode('utf8'),
+            DocTestMatches("""...test: ...failing
 ...failure: ...failing...""", doctest.ELLIPSIS))
 
     def test_with_subunit_no_failures_exit_0(self):
@@ -96,15 +88,12 @@ class TestCommand(ResourcedTestCase):
         repo = cmd.repository_factory.initialise(ui.here)
         inserter = repo.get_inserter()
         inserter.startTestRun()
-        class Cases(ResourcedTestCase):
-            def ok(self):
-                pass
-        Cases('ok').run(inserter)
+        inserter.status(test_id='ok', test_status='success')
         inserter.stopTestRun()
         self.assertEqual(0, cmd.execute())
         self.assertEqual(1, len(ui.outputs))
         self.assertEqual('stream', ui.outputs[0][0])
-        self.assertThat(ui.outputs[0][1], Equals(''))
+        self.assertThat(ui.outputs[0][1], Equals(_b('')))
 
     def test_with_list_shows_list_of_tests(self):
         ui, cmd = self.get_test_ui_and_cmd(options=[('list', True)])
@@ -112,22 +101,15 @@ class TestCommand(ResourcedTestCase):
         repo = cmd.repository_factory.initialise(ui.here)
         inserter = repo.get_inserter()
         inserter.startTestRun()
-        class Cases(ResourcedTestCase):
-            def failing1(self):
-                self.fail('foo')
-            def failing2(self):
-                self.fail('bar')
-            def ok(self):
-                pass
-        Cases('failing1').run(inserter)
-        Cases('ok').run(inserter)
-        Cases('failing2').run(inserter)
+        inserter.status(test_id='failing1', test_status='fail')
+        inserter.status(test_id='ok', test_status='success')
+        inserter.status(test_id='failing2', test_status='fail')
         inserter.stopTestRun()
         self.assertEqual(1, cmd.execute(), ui.outputs)
         self.assertEqual(1, len(ui.outputs))
         self.assertEqual('tests', ui.outputs[0][0])
         self.assertEqual(
-            set([Cases('failing1').id(), Cases('failing2').id()]),
+            set(['failing1', 'failing2']),
             set([test.id() for test in ui.outputs[0][1]]))
 
     def test_uses_get_failing(self):
@@ -139,13 +121,8 @@ class TestCommand(ResourcedTestCase):
             repo = open(url)
             inserter = repo.get_inserter()
             inserter.startTestRun()
-            class Cases(ResourcedTestCase):
-                def failing(self):
-                    self.fail('foo')
-                def ok(self):
-                    pass
-            Cases('failing').run(inserter)
-            Cases('ok').run(inserter)
+            inserter.status(test_id='failing', test_status='fail')
+            inserter.status(test_id='ok', test_status='success')
             inserter.stopTestRun()
             orig = repo.get_failing
             def get_failing():
