@@ -14,16 +14,21 @@
 
 """Tests for the run command."""
 
+from io import BytesIO
 import os.path
 from subprocess import PIPE
 import tempfile
 
+from extras import try_import
 from fixtures import (
     Fixture,
     MonkeyPatch,
     )
+import subunit
+v2_avail = try_import('subunit.ByteStreamToStreamResult')
 from subunit import RemotedTestCase
 from testscenarios.scenarios import multiply_scenarios
+from testtools.compat import _b
 from testtools.matchers import (
     Equals,
     MatchesException,
@@ -61,21 +66,18 @@ class TestCommand(ResourcedTestCase):
     def config_path(self):
         return os.path.join(self.tempdir, '.testr.conf')
 
-    def set_config(self, bytes):
-        stream = file(self.config_path(), 'wb')
-        try:
-            stream.write(bytes)
-        finally:
-            stream.close()
+    def set_config(self, text):
+        with open(self.config_path(), 'wt') as stream:
+            stream.write(text)
 
     def setup_repo(self, cmd, ui, failures=True):
         repo = cmd.repository_factory.initialise(ui.here)
         inserter = repo.get_inserter()
         inserter.startTestRun()
-        make_test('passing', True).run(inserter)
+        inserter.status(test_id='passing', test_status='success')
         if failures:
-            make_test('failing1', False).run(inserter)
-            make_test('failing2', False).run(inserter)
+            inserter.status(test_id='failing1', test_status='fail')
+            inserter.status(test_id='failing2', test_status='fail')
         inserter.stopTestRun()
 
     def test_no_config_file_errors(self):
@@ -296,8 +298,16 @@ class TestCommand(ResourcedTestCase):
             cmd.repository_factory.repos[ui.here].get_test_run(1)._partial)
 
     def test_load_failure_exposed(self):
+        if v2_avail:
+            buffer = BytesIO()
+            stream = subunit.StreamResultToBytes(buffer)
+            stream.status(test_id='foo', test_status='inprogress')
+            stream.status(test_id='foo', test_status='fail')
+            subunit_bytes = buffer.getvalue()
+        else:
+            subunit_bytes = b'test: foo\nfailure: foo\n'
         ui, cmd = self.get_test_ui_and_cmd(options=[('quiet', True),],
-            proc_outputs=['test: foo\nfailure: foo\n'])
+            proc_outputs=[subunit_bytes])
         cmd.repository_factory = memory.RepositoryFactory()
         self.setup_repo(cmd, ui)
         self.set_config('[DEFAULT]\ntest_command=foo\n')
@@ -306,8 +316,16 @@ class TestCommand(ResourcedTestCase):
         self.assertEqual(1, result)
 
     def test_process_exit_code_nonzero_causes_synthetic_error_test(self):
+        if v2_avail:
+            buffer = BytesIO()
+            stream = subunit.StreamResultToBytes(buffer)
+            stream.status(test_id='foo', test_status='inprogress')
+            stream.status(test_id='foo', test_status='success')
+            subunit_bytes = buffer.getvalue()
+        else:
+            subunit_bytes = b'test: foo\nsuccess: foo\n'
         ui, cmd = self.get_test_ui_and_cmd(options=[('quiet', True),],
-            proc_outputs=['test: foo\nsuccess: foo\n'],
+            proc_outputs=[subunit_bytes],
             proc_results=[2])
             # 2 is non-zero, and non-zero triggers the behaviour of exiting
             # with 1 - but we want to see that it doesn't pass-through the
@@ -318,14 +336,12 @@ class TestCommand(ResourcedTestCase):
         result = cmd.execute()
         self.assertEqual(1, result)
         run = cmd.repository_factory.repos[ui.here].get_test_run(1)
-        self.assertEqual([
-            Wildcard,
-            ('Error', RemotedTestCase('process-returncode'), Wildcard)],
-            run._outcomes)
+        self.assertEqual([Wildcard, 'fail'],
+            [test['status'] for test in run._tests])
 
     def test_regex_test_filter(self):
         ui, cmd = self.get_test_ui_and_cmd(args=('ab.*cd', '--', 'bar', 'quux'))
-        ui.proc_outputs = ['ab-cd\nefgh\n']
+        ui.proc_outputs = [_b('ab-cd\nefgh\n')]
         cmd.repository_factory = memory.RepositoryFactory()
         self.setup_repo(cmd, ui)
         self.set_config(
@@ -374,10 +390,25 @@ class TestCommand(ResourcedTestCase):
 
     def test_until_failure(self):
         ui, cmd = self.get_test_ui_and_cmd(options=[('until_failure', True)])
+        if v2_avail:
+            buffer = BytesIO()
+            stream = subunit.StreamResultToBytes(buffer)
+            stream.status(test_id='foo', test_status='inprogress')
+            stream.status(test_id='foo', test_status='success')
+            subunit_bytes1 = buffer.getvalue()
+            buffer.seek(0)
+            buffer.truncate()
+            stream.status(test_id='foo', test_status='inprogress')
+            stream.status(test_id='foo', test_status='fail')
+            subunit_bytes2 = buffer.getvalue()
+        else:
+            subunit_bytes1 = b'test: foo\nsuccess: foo\n'
+            subunit_bytes2 = b'test: foo\nfailure: foo\n'
         ui.proc_outputs = [
-            'test: foo\nsuccess: foo\n', # stream one, works
-            'test: foo\nfailure: foo\n' # stream two, fails
+            subunit_bytes1, # stream one, works
+            subunit_bytes2, # stream two, fails
             ]
+        ui.require_proc_stdout = True
         cmd.repository_factory = memory.RepositoryFactory()
         self.setup_repo(cmd, ui)
         self.set_config(
@@ -391,7 +422,7 @@ class TestCommand(ResourcedTestCase):
             ('popen', (expected_cmd,),
              {'shell': True, 'stdin': PIPE, 'stdout': PIPE}),
             ('results', Wildcard),
-            ('summary', True, 1, -2, Wildcard, None, [('id', 1, None)]),
+            ('summary', True, 1, -2, Wildcard, Wildcard, [('id', 1, None)]),
             ('values', [('running', expected_cmd)]),
             ('popen', (expected_cmd,),
              {'shell': True, 'stdin': PIPE, 'stdout': PIPE}),
@@ -429,7 +460,7 @@ def readline(stream):
 
 
 def readlines(stream):
-    return ''.join(stream.readlines())
+    return _b('').join(stream.readlines())
 
 
 def accumulate(stream, reader):
@@ -438,7 +469,7 @@ def accumulate(stream, reader):
     while content:
         accumulator.append(content)
         content = reader(stream)
-    return ''.join(accumulator)
+    return _b('').join(accumulator)
 
 
 class TestReturnCodeToSubunit(ResourcedTestCase):
@@ -449,8 +480,8 @@ class TestReturnCodeToSubunit(ResourcedTestCase):
          ('readline', dict(reader=readline)),
          ('readlines', dict(reader=readlines)),
          ],
-        [('noeol', dict(stdout='foo\nbar')),
-         ('trailingeol', dict(stdout='foo\nbar\n'))])
+        [('noeol', dict(stdout=_b('foo\nbar'))),
+         ('trailingeol', dict(stdout=_b('foo\nbar\n')))])
 
     def test_returncode_0_no_change(self):
         proc = ProcessModel(None)
@@ -467,7 +498,15 @@ class TestReturnCodeToSubunit(ResourcedTestCase):
         proc.returncode = 1
         stream = run.ReturnCodeToSubunit(proc)
         content = accumulate(stream, self.reader)
-        self.assertEqual(
-            'foo\nbar\ntest: process-returncode\n'
-            'error: process-returncode [\n returncode 1\n]\n',
-            content)
+        if v2_avail:
+            buffer = BytesIO()
+            buffer.write(b'foo\nbar\n')
+            stream = subunit.StreamResultToBytes(buffer)
+            stream.status(test_id='process-returncode', test_status='fail',
+                file_name='traceback', mime_type='test/plain;charset=utf8',
+                file_bytes=b'returncode 1')
+            expected_content = buffer.getvalue()
+        else:
+            expected_content = _b('foo\nbar\ntest: process-returncode\n'
+                'failure: process-returncode [\n returncode 1\n]\n')
+        self.assertEqual(expected_content, content)
