@@ -14,10 +14,15 @@
 
 """Tests for the testcommand module."""
 
+from io import BytesIO
 import os.path
 import optparse
 import re
 
+from extras import try_import
+import subunit
+v2_avail = try_import('subunit.ByteStreamToStreamResult')
+from testtools.compat import _b
 from testtools.matchers import (
     Equals,
     MatchesAny,
@@ -67,10 +72,10 @@ class TestTestCommand(ResourcedTestCase):
     def config_path(self):
         return os.path.join(self.tempdir, '.testr.conf')
 
-    def set_config(self, bytes):
-        stream = file(self.config_path(), 'wb')
+    def set_config(self, text):
+        stream = open(self.config_path(), 'wt')
         try:
-            stream.write(bytes)
+            stream.write(text)
         finally:
             stream.close()
 
@@ -103,7 +108,7 @@ class TestTestCommand(ResourcedTestCase):
         self.set_config(
             '[DEFAULT]\ntest_command=foo\n'
             'instance_dispose=bar $INSTANCE_IDS\n')
-        command._instances.update(['baz', 'quux'])
+        command._instances.update([_b('baz'), _b('quux')])
         command.cleanUp()
         command.setUp()
         self.assertEqual([
@@ -117,7 +122,7 @@ class TestTestCommand(ResourcedTestCase):
         self.set_config(
             '[DEFAULT]\ntest_command=foo\n'
             'instance_dispose=bar $INSTANCE_IDS\n')
-        command._instances.update(['baz', 'quux'])
+        command._instances.update([_b('baz'), _b('quux')])
         self.assertThat(command.cleanUp,
             raises(ValueError('Disposing of instances failed, return 1')))
         command.setUp()
@@ -142,7 +147,7 @@ class TestTestCommand(ResourcedTestCase):
         try:
             fixture.setUp()
             list_file_path = fixture.list_file_name
-            source = open(list_file_path, 'rb')
+            source = open(list_file_path, 'rt')
             try:
                 list_file_content = source.read()
             finally:
@@ -180,7 +185,15 @@ class TestTestCommand(ResourcedTestCase):
 
     def test_get_run_command_default_and_list_expands(self):
         ui, command = self.get_test_ui_and_cmd()
-        ui.proc_outputs = ['returned\nids\n']
+        if v2_avail:
+            buffer = BytesIO()
+            stream = subunit.StreamResultToBytes(buffer)
+            stream.status(test_id='returned', test_status='exists')
+            stream.status(test_id='ids', test_status='exists')
+            subunit_bytes = buffer.getvalue()
+        else:
+            subunit_bytes = _b('returned\nids\n')
+        ui.proc_outputs = [subunit_bytes]
         ui.options = optparse.Values()
         ui.options.parallel = True
         ui.options.concurrency = 2
@@ -228,7 +241,7 @@ class TestTestCommand(ResourcedTestCase):
         ui.here = self.tempdir
         cmd = run.run(ui)
         ui.set_command(cmd)
-        ui.proc_outputs = ['returned\ninstances\n']
+        ui.proc_outputs = [_b('returned\ninstances\n')]
         command = self.useFixture(TestCommand(ui, None))
         self.set_config(
             '[DEFAULT]\ntest_command=foo $LISTOPT $IDLIST\ntest_id_list_default=whoo yea\n'
@@ -237,7 +250,7 @@ class TestTestCommand(ResourcedTestCase):
             'instance_execute=quux $INSTANCE_ID -- $COMMAND\n')
         fixture = self.useFixture(command.get_run_command(test_ids=['1']))
         fixture.list_tests()
-        self.assertEqual(set(['returned', 'instances']), command._instances)
+        self.assertEqual(set([_b('returned'), _b('instances')]), command._instances)
         self.assertEqual(set([]), command._allocated_instances)
         self.assertThat(ui.outputs, MatchesAny(Equals([
             ('values', [('running', 'provision -c 2')]),
@@ -262,9 +275,9 @@ class TestTestCommand(ResourcedTestCase):
             'test_list_option=--list\n'
             'instance_execute=quux $INSTANCE_ID -- $COMMAND\n')
         fixture = self.useFixture(command.get_run_command())
-        command._instances.add('bar')
+        command._instances.add(_b('bar'))
         fixture.list_tests()
-        self.assertEqual(set(['bar']), command._instances)
+        self.assertEqual(set([_b('bar')]), command._instances)
         self.assertEqual(set([]), command._allocated_instances)
         self.assertEqual([
             ('values', [('running', 'quux bar -- foo --list whoo yea')]),
@@ -282,8 +295,16 @@ class TestTestCommand(ResourcedTestCase):
         self.assertEqual(expected_cmd, fixture.list_cmd)
 
     def test_list_tests_parsing(self):
+        if v2_avail:
+            buffer = BytesIO()
+            stream = subunit.StreamResultToBytes(buffer)
+            stream.status(test_id='returned', test_status='exists')
+            stream.status(test_id='ids', test_status='exists')
+            subunit_bytes = buffer.getvalue()
+        else:
+            subunit_bytes = _b('returned\nids\n')
         ui, command = self.get_test_ui_and_cmd()
-        ui.proc_outputs = ['returned\nids\n']
+        ui.proc_outputs = [subunit_bytes]
         self.set_config(
             '[DEFAULT]\ntest_command=foo $LISTOPT $IDLIST\ntest_id_list_default=whoo yea\n'
             'test_list_option=--list\n')
@@ -347,7 +368,7 @@ class TestTestCommand(ResourcedTestCase):
         ui, command = self.get_test_ui_and_cmd()
         self.set_config(
             '[DEFAULT]\ntest_command=foo $IDLIST\n')
-        command._instances.update(['foo', 'bar'])
+        command._instances.update([_b('foo'), _b('bar')])
         fixture = self.useFixture(command.get_run_command())
         procs = fixture.run_tests()
         self.assertEqual([
@@ -362,7 +383,7 @@ class TestTestCommand(ResourcedTestCase):
         self.set_config(
             '[DEFAULT]\ntest_command=foo $IDLIST\n'
             'instance_execute=quux $INSTANCE_ID -- $COMMAND\n')
-        command._instances.add('bar')
+        command._instances.add(_b('bar'))
         fixture = self.useFixture(command.get_run_command(test_ids=['1']))
         procs = fixture.run_tests()
         self.assertEqual([
@@ -371,13 +392,13 @@ class TestTestCommand(ResourcedTestCase):
              {'shell': True, 'stdin': -1, 'stdout': -1})],
             ui.outputs)
         # No --parallel, so the one instance should have been allocated.
-        self.assertEqual(set(['bar']), command._instances)
-        self.assertEqual(set(['bar']), command._allocated_instances)
+        self.assertEqual(set([_b('bar')]), command._instances)
+        self.assertEqual(set([_b('bar')]), command._allocated_instances)
         # And after the process is run, bar is returned for re-use.
         procs[0].stdout.read()
         procs[0].wait()
         self.assertEqual(0, procs[0].returncode)
-        self.assertEqual(set(['bar']), command._instances)
+        self.assertEqual(set([_b('bar')]), command._instances)
         self.assertEqual(set(), command._allocated_instances)
         
     def test_run_tests_allocated_instances_skipped(self):
@@ -385,8 +406,8 @@ class TestTestCommand(ResourcedTestCase):
         self.set_config(
             '[DEFAULT]\ntest_command=foo $IDLIST\n'
             'instance_execute=quux $INSTANCE_ID -- $COMMAND\n')
-        command._instances.update(['bar', 'baz'])
-        command._allocated_instances.add('baz')
+        command._instances.update([_b('bar'), _b('baz')])
+        command._allocated_instances.add(_b('baz'))
         fixture = self.useFixture(command.get_run_command(test_ids=['1']))
         procs = fixture.run_tests()
         self.assertEqual([
@@ -395,21 +416,21 @@ class TestTestCommand(ResourcedTestCase):
              {'shell': True, 'stdin': -1, 'stdout': -1})],
             ui.outputs)
         # No --parallel, so the one instance should have been allocated.
-        self.assertEqual(set(['bar', 'baz']), command._instances)
-        self.assertEqual(set(['bar', 'baz']), command._allocated_instances)
+        self.assertEqual(set([_b('bar'), _b('baz')]), command._instances)
+        self.assertEqual(set([_b('bar'), _b('baz')]), command._allocated_instances)
         # And after the process is run, bar is returned for re-use.
         procs[0].wait()
         procs[0].stdout.read()
         self.assertEqual(0, procs[0].returncode)
-        self.assertEqual(set(['bar', 'baz']), command._instances)
-        self.assertEqual(set(['baz']), command._allocated_instances)
+        self.assertEqual(set([_b('bar'), _b('baz')]), command._instances)
+        self.assertEqual(set([_b('baz')]), command._allocated_instances)
 
     def test_run_tests_list_file_in_FILES(self):
         ui, command = self.get_test_ui_and_cmd()
         self.set_config(
             '[DEFAULT]\ntest_command=foo $IDFILE\n'
             'instance_execute=quux $INSTANCE_ID $FILES -- $COMMAND\n')
-        command._instances.add('bar')
+        command._instances.add(_b('bar'))
         fixture = self.useFixture(command.get_run_command(test_ids=['1']))
         list_file = fixture.list_file_name
         procs = fixture.run_tests()
@@ -420,12 +441,12 @@ class TestTestCommand(ResourcedTestCase):
              {'shell': True, 'stdin': -1, 'stdout': -1})],
             ui.outputs)
         # No --parallel, so the one instance should have been allocated.
-        self.assertEqual(set(['bar']), command._instances)
-        self.assertEqual(set(['bar']), command._allocated_instances)
+        self.assertEqual(set([_b('bar')]), command._instances)
+        self.assertEqual(set([_b('bar')]), command._allocated_instances)
         # And after the process is run, bar is returned for re-use.
         procs[0].stdout.read()
         self.assertEqual(0, procs[0].returncode)
-        self.assertEqual(set(['bar']), command._instances)
+        self.assertEqual(set([_b('bar')]), command._instances)
         self.assertEqual(set(), command._allocated_instances)
 
     def test_filter_tags_parsing(self):
@@ -435,7 +456,7 @@ class TestTestCommand(ResourcedTestCase):
 
     def test_callout_concurrency(self):
         ui, command = self.get_test_ui_and_cmd()
-        ui.proc_outputs = ['4']
+        ui.proc_outputs = [_b('4')]
         self.set_config(
             '[DEFAULT]\ntest_run_concurrency=probe\n'
             'test_command=foo\n')
@@ -514,8 +535,16 @@ class TestTestCommand(ResourcedTestCase):
             log._events)
 
     def test_filter_tests_by_regex_only(self):
+        if v2_avail:
+            buffer = BytesIO()
+            stream = subunit.StreamResultToBytes(buffer)
+            stream.status(test_id='returned', test_status='exists')
+            stream.status(test_id='ids', test_status='exists')
+            subunit_bytes = buffer.getvalue()
+        else:
+            subunit_bytes = _b('returned\nids\n')
         ui, command = self.get_test_ui_and_cmd()
-        ui.proc_outputs = ['returned\nids\n']
+        ui.proc_outputs = [subunit_bytes]
         self.set_config(
             '[DEFAULT]\ntest_command=foo $LISTOPT $IDLIST\ntest_id_list_default=whoo yea\n'
             'test_list_option=--list\n')
@@ -525,7 +554,7 @@ class TestTestCommand(ResourcedTestCase):
 
     def test_filter_tests_by_regex_supplied_ids(self):
         ui, command = self.get_test_ui_and_cmd()
-        ui.proc_outputs = ['returned\nids\n']
+        ui.proc_outputs = [_b('returned\nids\n')]
         self.set_config(
             '[DEFAULT]\ntest_command=foo $LISTOPT $IDLIST\ntest_id_list_default=whoo yea\n'
             'test_list_option=--list\n')
