@@ -16,7 +16,7 @@
 """Tests for UI support logic and the UI contract."""
 
 import doctest
-from cStringIO import StringIO
+from io import BytesIO, StringIO, TextIOWrapper
 import optparse
 import os
 import sys
@@ -24,7 +24,9 @@ from textwrap import dedent
 
 from fixtures import EnvironmentVariable
 import subunit
+import testtools
 from testtools import TestCase
+from testtools.compat import _b, _u
 from testtools.matchers import (
     DocTestMatches,
     MatchesException,
@@ -38,7 +40,7 @@ from testrepository.tests import ResourcedTestCase, StubTestCommand
 
 
 def get_test_ui_and_cmd(options=(), args=()):
-    stdout = StringIO()
+    stdout = TextIOWrapper(BytesIO(), 'utf8', line_buffering=True)
     stdin = StringIO()
     stderr = StringIO()
     argv = list(args)
@@ -59,15 +61,15 @@ class TestCLIUI(ResourcedTestCase):
         self.useFixture(EnvironmentVariable('TESTR_PDB'))
 
     def test_construct(self):
-        stdout = StringIO()
-        stdin = StringIO()
-        stderr = StringIO()
+        stdout = BytesIO()
+        stdin = BytesIO()
+        stderr = BytesIO()
         cli.UI([], stdin, stdout, stderr)
 
     def test_stream_comes_from_stdin(self):
-        stdout = StringIO()
-        stdin = StringIO('foo\n')
-        stderr = StringIO()
+        stdout = BytesIO()
+        stdin = BytesIO(_b('foo\n'))
+        stderr = BytesIO()
         ui = cli.UI([], stdin, stdout, stderr)
         cmd = commands.Command(ui)
         cmd.input_streams = ['subunit']
@@ -75,12 +77,12 @@ class TestCLIUI(ResourcedTestCase):
         results = []
         for stream in ui.iter_streams('subunit'):
             results.append(stream.read())
-        self.assertEqual(['foo\n'], results)
+        self.assertEqual([_b('foo\n')], results)
 
     def test_dash_d_sets_here_option(self):
-        stdout = StringIO()
-        stdin = StringIO('foo\n')
-        stderr = StringIO()
+        stdout = BytesIO()
+        stdin = BytesIO(_b('foo\n'))
+        stderr = BytesIO()
         ui = cli.UI(['-d', '/nowhere/'], stdin, stdout, stderr)
         cmd = commands.Command(ui)
         ui.set_command(cmd)
@@ -112,7 +114,7 @@ class TestCLIUI(ResourcedTestCase):
             fooo
             """)
         stdout = StringIO()
-        stdin = StringIO('c\n')
+        stdin = StringIO(_u('c\n'))
         stderr = StringIO()
         ui = cli.UI([], stdin, stdout, stderr)
         ui.output_error(err_tuple)
@@ -121,18 +123,21 @@ class TestCLIUI(ResourcedTestCase):
 
     def test_outputs_rest_to_stdout(self):
         ui, cmd = get_test_ui_and_cmd()
-        ui.output_rest('topic\n=====\n')
-        self.assertEqual('topic\n=====\n', ui._stdout.getvalue())
+        ui.output_rest(_u('topic\n=====\n'))
+        self.assertEqual(_b('topic\n=====\n'), ui._stdout.buffer.getvalue())
 
     def test_outputs_results_to_stdout(self):
         ui, cmd = get_test_ui_and_cmd()
         class Case(ResourcedTestCase):
             def method(self):
                 self.fail('quux')
-        result = ui.make_result(lambda: None, StubTestCommand())
-        Case('method').run(result)
-        self.assertThat(ui._stdout.getvalue(),DocTestMatches(
-            """======================================================================
+        result, summary = ui.make_result(lambda: None, StubTestCommand())
+        result.startTestRun()
+        Case('method').run(testtools.ExtendedToStreamDecorator(result))
+        result.stopTestRun()
+        self.assertThat(ui._stdout.buffer.getvalue().decode('utf8'),
+            DocTestMatches("""\
+======================================================================
 FAIL: testrepository.tests.ui.test_cli.Case.method
 ----------------------------------------------------------------------
 ...Traceback (most recent call last):...
@@ -143,21 +148,21 @@ AssertionError: quux...
 
     def test_outputs_stream_to_stdout(self):
         ui, cmd = get_test_ui_and_cmd()
-        stream = StringIO("Foo \n bar")
+        stream = BytesIO(_b("Foo \n bar"))
         ui.output_stream(stream)
-        self.assertEqual("Foo \n bar", ui._stdout.getvalue())
+        self.assertEqual(_b("Foo \n bar"), ui._stdout.buffer.getvalue())
 
     def test_outputs_tables_to_stdout(self):
         ui, cmd = get_test_ui_and_cmd()
         ui.output_table([('foo', 1), ('b', 'quux')])
-        self.assertEqual('foo  1\n---  ----\nb    quux\n',
-            ui._stdout.getvalue())
+        self.assertEqual(_b('foo  1\n---  ----\nb    quux\n'),
+            ui._stdout.buffer.getvalue())
 
     def test_outputs_tests_to_stdout(self):
         ui, cmd = get_test_ui_and_cmd()
         ui.output_tests([self, self.__class__('test_construct')])
         self.assertThat(
-            ui._stdout.getvalue(),
+            ui._stdout.buffer.getvalue().decode('utf8'),
             DocTestMatches(
                 '...TestCLIUI.test_outputs_tests_to_stdout\n'
                 '...TestCLIUI.test_construct\n', doctest.ELLIPSIS))
@@ -165,14 +170,15 @@ AssertionError: quux...
     def test_outputs_values_to_stdout(self):
         ui, cmd = get_test_ui_and_cmd()
         ui.output_values([('foo', 1), ('bar', 'quux')])
-        self.assertEqual('foo=1, bar=quux\n', ui._stdout.getvalue())
+        self.assertEqual(_b('foo=1, bar=quux\n'), ui._stdout.buffer.getvalue())
 
     def test_outputs_summary_to_stdout(self):
         ui, cmd = get_test_ui_and_cmd()
         summary = [True, 1, None, 2, None, []]
         expected_summary = ui._format_summary(*summary)
         ui.output_summary(*summary)
-        self.assertEqual("%s\n" % (expected_summary,), ui._stdout.getvalue())
+        self.assertEqual(_b("%s\n" % (expected_summary,)),
+            ui._stdout.buffer.getvalue())
 
     def test_parse_error_goes_to_stderr(self):
         stdout = StringIO()
@@ -194,9 +200,9 @@ AssertionError: quux...
         self.assertEqual("Unexpected arguments: ['one']\n", stderr.getvalue())
 
     def test_parse_options_after_double_dash_are_arguments(self):
-        stdout = StringIO()
-        stdin = StringIO()
-        stderr = StringIO()
+        stdout = BytesIO()
+        stdin = BytesIO()
+        stderr = BytesIO()
         ui = cli.UI(['one', '--', '--two', 'three'], stdin, stdout, stderr)
         cmd = commands.Command(ui)
         cmd.args = [arguments.string.StringArgument('myargs', max=None),
@@ -213,9 +219,9 @@ AssertionError: quux...
         class CaptureArg(arguments.AbstractArgument):
             def _parse_one(self, arg):
                 return arg
-        stdout = StringIO()
-        stdin = StringIO()
-        stderr = StringIO()
+        stdout = BytesIO()
+        stdin = BytesIO()
+        stderr = BytesIO()
         ui = cli.UI(['one', '--', '--two', 'three'], stdin, stdout, stderr)
         cmd = commands.Command(ui)
         cmd.args = [CaptureArg('args', max=None)]
@@ -226,21 +232,6 @@ AssertionError: quux...
         ui, cmd = get_test_ui_and_cmd(options=[('subunit', True)])
         self.assertEqual(True, ui.options.subunit)
 
-    def test_subunit_output_with_full_results(self):
-        # When --full-results is passed in, successes are included in the
-        # subunit output.
-        ui, cmd = get_test_ui_and_cmd(options=[('subunit', True)])
-        stream = StringIO()
-        subunit_result = subunit.TestProtocolClient(stream)
-        subunit_result.startTest(self)
-        subunit_result.addSuccess(self)
-        subunit_result.stopTest(self)
-        result = ui.make_result(lambda: None, StubTestCommand())
-        result.startTest(self)
-        result.addSuccess(self)
-        result.stopTest(self)
-        self.assertEqual(stream.getvalue(), ui._stdout.getvalue())
-
     def test_dash_dash_help_shows_help(self):
         stdout = StringIO()
         stdin = StringIO()
@@ -329,18 +320,16 @@ class TestCLITestResult(TestCase):
         except ZeroDivisionError:
             return sys.exc_info()
 
-    def make_result(self, stream=None, fullresults=False):
+    def make_result(self, stream=None, argv=None):
         if stream is None:
-            stream = StringIO()
-        argv = []
-        if fullresults:
-            argv.append('--full-results')
+            stream = BytesIO()
+        argv = argv or []
         ui = cli.UI(argv, None, stream, None)
         cmd = commands.Command(ui)
-        if fullresults:
-            cmd.options = [optparse.Option(
-                "--full-results", action="store_true", default=False,
-                help="Show full results.")]
+        cmd.options = [
+            optparse.Option("--subunit", action="store_true",
+                default=False, help="Display results in subunit format."),
+            ]
         ui.set_command(cmd)
         return ui.make_result(lambda: None, StubTestCommand())
 
@@ -351,58 +340,68 @@ class TestCLITestResult(TestCase):
         cli.CLITestResult(cli.UI(None, None, None, None), stream, lambda: None)
         self.assertEqual('', stream.getvalue())
 
+    def _unwrap(self, result):
+        """Unwrap result to get to the CLI result object."""
+        return result.targets[0].decorated.decorated._results[1].decorated
+
     def test_format_error(self):
         # CLITestResult formats errors by giving them a big fat line, a title
         # made up of their 'label' and the name of the test, another different
         # big fat line, and then the actual error itself.
-        result = self.make_result(fullresults=True)
+        result = self._unwrap(self.make_result()[0])
         error = result._format_error('label', self, 'error text')
         expected = '%s%s: %s\n%s%s' % (
             result.sep1, 'label', self.id(), result.sep2, 'error text')
         self.assertThat(error, DocTestMatches(expected))
 
     def test_format_error_includes_tags(self):
-        result = self.make_result(fullresults=True)
-        result.tags(['foo'], ['bar'])
+        result1 = self.make_result()
+        result = self._unwrap(result1[0])
+        #result1.startTestRun()
+        #result1.status(test_id=self.id(), test_status='fail', eof=True,
+        #    test_tags=set(['foo']), file_name='traceback',
+        #    mime_type='test/plain;charset=utf8', file_bytes=b'error text')
+        result.tags(set(['foo']), set())
         error = result._format_error('label', self, 'error text')
         expected = '%s%s: %s\ntags: foo\n%s%s' % (
             result.sep1, 'label', self.id(), result.sep2, 'error text')
         self.assertThat(error, DocTestMatches(expected))
 
-    def test_addError_outputs_error(self):
-        # CLITestResult.addError outputs the given error immediately to the
-        # stream.
+    def test_addFail_outputs_error(self):
+        # CLITestResult.status test_status='fail' outputs the given error
+        # immediately to the stream.
         stream = StringIO()
-        result = self.make_result(stream, fullresults=True)
+        result = self.make_result(stream)[0]
         error = self.make_exc_info()
-        error_text = result._err_details_to_string(self, error)
-        result.addError(self, error)
+        error_text = 'foo\nbar\n'
+        result.startTestRun()
+        result.status(test_id=self.id(), test_status='fail', eof=True,
+            file_name='traceback', mime_type='text/plain;charset=utf8',
+            file_bytes=error_text.encode('utf8'))
+        result1 = self._unwrap(result)
         self.assertThat(
             stream.getvalue(),
-            DocTestMatches(result._format_error('ERROR', self, error_text)))
-
-    def test_addFailure_outputs_failure(self):
-        # CLITestResult.addFailure outputs the given error immediately to the
-        # stream.
-        stream = StringIO()
-        result = self.make_result(stream, fullresults=True)
-        error = self.make_exc_info()
-        error_text = result._err_details_to_string(self, error)
-        result.addFailure(self, error)
-        self.assertThat(
-            stream.getvalue(),
-            DocTestMatches(result._format_error('FAIL', self, error_text)))
+            DocTestMatches(result1._format_error('FAIL', self, error_text)))
 
     def test_addFailure_handles_string_encoding(self):
         # CLITestResult.addFailure outputs the given error handling non-ascii
         # characters.
-        stream = StringIO()
-        result = self.make_result(stream, fullresults=True)
-        class MyError(ValueError):
-            def __unicode__(self):
-                return u'\u201c'
-        error = (MyError, MyError(), None)
-        result.addFailure(self, error)
+        # Lets say we have bytes output, not string for some reason.
+        stream = BytesIO()
+        result = self.make_result(stream)[0]
+        result.startTestRun()
+        result.status(test_id='foo', test_status='fail', file_name='traceback',
+            mime_type='text/plain;charset=utf8',
+            file_bytes=b'-->\xe2\x80\x9c<--', eof=True)
+        pattern = _u("...-->?<--...")
         self.assertThat(
-            stream.getvalue(),
-            DocTestMatches("...MyError: ?", doctest.ELLIPSIS))
+            stream.getvalue().decode('utf8'),
+            DocTestMatches(pattern, doctest.ELLIPSIS))
+
+    def test_subunit_output(self):
+        bytestream = BytesIO()
+        stream = TextIOWrapper(bytestream, 'utf8', line_buffering=True)
+        result = self.make_result(stream, argv=['--subunit'])[0]
+        result.startTestRun()
+        result.stopTestRun()
+        self.assertEqual(b'', bytestream.getvalue())
