@@ -14,9 +14,11 @@
 
 """Am object based UI for testrepository."""
 
-from cStringIO import StringIO
+from io import BytesIO
 import optparse
 
+import testtools
+
 from testrepository import ui
 
 
@@ -26,8 +28,8 @@ class ProcessModel(object):
     def __init__(self, ui):
         self.ui = ui
         self.returncode = 0
-        self.stdin = StringIO()
-        self.stdout = StringIO()
+        self.stdin = BytesIO()
+        self.stdout = BytesIO()
 
     def communicate(self):
         self.ui.outputs.append(('communicate',))
@@ -63,6 +65,11 @@ class TestResultModel(ui.BaseUITestResult):
     def stopTest(self, test):
         self._suite.recordResult('stopTest', test)
 
+    def status(self, test_id=None, test_status=None, test_tags=None,
+        runnable=True, file_name=None, file_bytes=None, eof=False,
+        mime_type=None, route_code=None, timestamp=None):
+        self._suite.recordResult('status', test_id, test_status)
+
     def addError(self, test, *args):
         super(TestResultModel, self).addError(test, *args)
         self._suite.recordResult('addError', test, *args)
@@ -103,6 +110,8 @@ class UI(ui.AbstractUI):
         self.input_streams = {}
         if input_streams:
             for stream_type, stream_value in input_streams:
+                if isinstance(stream_value, str) and str is not bytes:
+                    raise Exception('bad stream_value')
                 self.input_streams.setdefault(stream_type, []).append(
                     stream_value)
         self.here = here
@@ -111,6 +120,7 @@ class UI(ui.AbstractUI):
         # Could take parsed args, but for now this is easier.
         self.unparsed_args = args
         self.proc_outputs = list(proc_outputs)
+        self.require_proc_stdout = False
         self.proc_results = list(proc_results)
 
     def _check_cmd(self):
@@ -143,11 +153,14 @@ class UI(ui.AbstractUI):
             if getattr(stream_value, 'read', None):
                 yield stream_value
             else:
-                yield StringIO(stream_value)
+                yield BytesIO(stream_value)
 
     def make_result(self, get_id, test_command, previous_run=None):
-        return test_command.make_result(
-            TestResultModel(self, get_id, previous_run))
+        result = testtools.ExtendedToStreamDecorator(
+            testtools.StreamToExtendedDecorator(
+            test_command.make_result(
+            TestResultModel(self, get_id, previous_run))))
+        return result, result
 
     def output_error(self, error_tuple):
         self.outputs.append(('error', error_tuple))
@@ -177,7 +190,9 @@ class UI(ui.AbstractUI):
         self.outputs.append(('popen', args, kwargs))
         result = ProcessModel(self)
         if self.proc_outputs:
-            result.stdout = StringIO(self.proc_outputs.pop(0))
+            result.stdout = BytesIO(self.proc_outputs.pop(0))
+        elif self.require_proc_stdout:
+            raise Exception("No process output available")
         if self.proc_results:
             result.returncode = self.proc_results.pop(0)
         return result
